@@ -4,14 +4,14 @@ use {
     solana_clap_utils::{
         input_parsers::{pubkey_of, pubkeys_of, value_of},
         input_validators::{
-            is_pubkey, is_pubkey_or_keypair, is_slot, is_url_or_moniker,
+            is_parsable, is_pubkey, is_pubkey_or_keypair, is_slot, is_url_or_moniker,
             normalize_to_url_if_moniker,
         },
     },
     solana_client::rpc_client::RpcClient,
     solana_core::tower_storage::FileTowerStorage,
     solana_faucet::faucet::{run_local_faucet_with_port, FAUCET_PORT},
-    solana_rpc::rpc::JsonRpcConfig,
+    solana_rpc::{rpc::JsonRpcConfig, rpc_pubsub_service::PubSubConfig},
     solana_sdk::{
         account::AccountSharedData,
         clock::Slot,
@@ -154,6 +154,12 @@ fn main() {
                 .validator(solana_validator::port_validator)
                 .help("Enable JSON RPC on this port, and the next port for the RPC websocket"),
         )
+        .arg(
+            Arg::with_name("rpc_pubsub_enable_vote_subscription")
+                .long("rpc-pubsub-enable-vote-subscription")
+                .takes_value(false)
+                .help("Enable the unstable RPC PubSub `voteSubscribe` subscription"),
+        )
         .arg(
             Arg::with_name("bpf_program")
                 .long("bpf-program")
@@ -186,6 +192,14 @@ fn main() {
                 .takes_value(false)
                 .help("Disable the just-in-time compiler and instead use the interpreter for BPF. Windows always disables JIT."),
         )
+        .arg(
+            Arg::with_name("ticks_per_slot")
+                .long("ticks-per-slot")
+                .value_name("TICKS")
+                .validator(is_parsable::<u64>)
+                .takes_value(true)
+                .help("The number of ticks in a slot"),
+        )
         .arg(
             Arg::with_name("slots_per_epoch")
                 .long("slots-per-epoch")
@@ -310,6 +324,15 @@ fn main() {
                 .long("no-accounts-db-caching")
                 .help("Disables accounts caching"),
         )
+        .arg(
+            Arg::with_name("deactivate_feature")
+                .long("deactivate-feature")
+                .takes_value(true)
+                .value_name("FEATURE_PUBKEY")
+                .validator(is_pubkey)
+                .multiple(true)
+                .help("deactivate this feature in genesis.")
+        )
         .get_matches();
 
     let output = if matches.is_present("quiet") {
@@ -395,7 +418,9 @@ fn main() {
         });
 
     let rpc_port = value_t_or_exit!(matches, "rpc_port", u16);
+    let enable_vote_subscription = matches.is_present("rpc_pubsub_enable_vote_subscription");
     let faucet_port = value_t_or_exit!(matches, "faucet_port", u16);
+    let ticks_per_slot = value_t!(matches, "ticks_per_slot", u64).ok();
     let slots_per_epoch = value_t!(matches, "slots_per_epoch", Slot).ok();
     let gossip_host = matches.value_of("gossip_host").map(|gossip_host| {
         solana_net_utils::parse_host(gossip_host).unwrap_or_else(|err| {
@@ -532,14 +557,18 @@ fn main() {
         });
     }
 
+    let features_to_deactivate = pubkeys_of(&matches, "deactivate_feature").unwrap_or_default();
+
     if TestValidatorGenesis::ledger_exists(&ledger_path) {
         for (name, long) in &[
             ("bpf_program", "--bpf-program"),
             ("clone_account", "--clone"),
             ("account", "--account"),
             ("mint_address", "--mint"),
+            ("ticks_per_slot", "--ticks-per-slot"),
             ("slots_per_epoch", "--slots-per-epoch"),
             ("faucet_sol", "--faucet-sol"),
+            ("deactivate_feature", "--deactivate-feature"),
         ] {
             if matches.is_present(name) {
                 println!("{} argument ignored, ledger already exists", long);
@@ -599,12 +628,17 @@ fn main() {
             enable_rpc_transaction_history: true,
             enable_cpi_and_log_storage: true,
             faucet_addr,
-            ..JsonRpcConfig::default()
+            ..JsonRpcConfig::default_for_test()
+        })
+        .pubsub_config(PubSubConfig {
+            enable_vote_subscription,
+            ..PubSubConfig::default()
         })
         .bpf_jit(!matches.is_present("no_bpf_jit"))
         .rpc_port(rpc_port)
         .add_programs_with_path(&programs_to_load)
-        .add_accounts_from_json_files(&accounts_to_load);
+        .add_accounts_from_json_files(&accounts_to_load)
+        .deactivate_features(&features_to_deactivate);
 
     if !accounts_to_clone.is_empty() {
         genesis.clone_accounts(
@@ -619,6 +653,10 @@ fn main() {
         genesis.warp_slot(warp_slot);
     }
 
+    if let Some(ticks_per_slot) = ticks_per_slot {
+        genesis.ticks_per_slot(ticks_per_slot);
+    }
+
     if let Some(slots_per_epoch) = slots_per_epoch {
         genesis.epoch_schedule(EpochSchedule::custom(
             slots_per_epoch,
